@@ -209,7 +209,7 @@ _Bool SPVM_TYPE_is_string(SPVM_COMPILER* compiler, SPVM_TYPE* type) {
 _Bool SPVM_TYPE_is_package(SPVM_COMPILER* compiler, SPVM_TYPE* type) {
   (void)compiler;
   
-  _Bool is_package = type->dimension == 0 && type->code > SPVM_TYPE_C_CODE_DOUBLE;
+  _Bool is_package = type->dimension == 0 && type->code >= SPVM_TYPE_C_CODE_STRING;
   
   return is_package;
 }
@@ -217,7 +217,9 @@ _Bool SPVM_TYPE_is_package(SPVM_COMPILER* compiler, SPVM_TYPE* type) {
 _Bool SPVM_TYPE_is_array_numeric(SPVM_COMPILER* compiler, SPVM_TYPE* type) {
   (void)compiler;
   
-  if (type && type->code >= SPVM_TYPE_C_CODE_BYTE_ARRAY && type->code <= SPVM_TYPE_C_CODE_DOUBLE_ARRAY)
+  assert(type);
+  
+  if (type->code >= SPVM_TYPE_C_CODE_BYTE_ARRAY && type->code <= SPVM_TYPE_C_CODE_DOUBLE_ARRAY)
   {
     return 1;
   }
@@ -229,7 +231,9 @@ _Bool SPVM_TYPE_is_array_numeric(SPVM_COMPILER* compiler, SPVM_TYPE* type) {
 _Bool SPVM_TYPE_is_integral(SPVM_COMPILER* compiler, SPVM_TYPE* type) {
   (void)compiler;
   
-  if (type && type->code >= SPVM_TYPE_C_CODE_BYTE && type->code <= SPVM_TYPE_C_CODE_LONG) {
+  assert(type);
+  
+  if (type->code >= SPVM_TYPE_C_CODE_BYTE && type->code <= SPVM_TYPE_C_CODE_LONG) {
     return 1;
   }
   else {
@@ -240,7 +244,20 @@ _Bool SPVM_TYPE_is_integral(SPVM_COMPILER* compiler, SPVM_TYPE* type) {
 _Bool SPVM_TYPE_is_numeric(SPVM_COMPILER* compiler, SPVM_TYPE* type) {
   (void)compiler;
   
-  if (type && type->code >= SPVM_TYPE_C_CODE_BYTE && type->code <= SPVM_TYPE_C_CODE_DOUBLE) {
+  assert(type);
+  
+  if (type->code >= SPVM_TYPE_C_CODE_BYTE && type->code <= SPVM_TYPE_C_CODE_DOUBLE) {
+    return 1;
+  }
+  else {
+    return 0;
+  }
+}
+
+_Bool SPVM_TYPE_is_object(SPVM_COMPILER* compiler, SPVM_TYPE* type) {
+  (void)compiler;
+  
+  if (type->code > SPVM_TYPE_C_CODE_DOUBLE) {
     return 1;
   }
   else {
